% startMAC.m
% an easy example call of the DA optimizer for MAC


%% load the data; each row represents an object, the columns are dimensions
load data.mat
imbin(data);


%% model hypothesis
% number of clusters
Nsrc    = 3;    
% degrees hypothesis ( a vector with all degrees of multi-assignments),
% where 0 means that objects are allowed to be only generated by noise, 1
% means membership to one cluster, 2 means membership to 2 clusters...
% ( here, we want zero-, single- and double-assignments )
Degrees = 0:2; 

% if this flag is true, then all combinations of the Nsrc clusters with the
% possible degrees Degrees are generated. If false only a fraction multFrac
% of Nsrc is used to generate combinations with Degree>1
useAllcombisFlag = true;
if ~useAllcombisFlag
    multFrac = 0.1;
end

%% set MAC params
% loads a standard configuration of the DA optimizer
% This must be done AFTER the parameters Nsrc and Degrees are set!
setMACparams;


%% run MAC optimizer
[ MinRisk_assignSets, betas, epsilon, r, currisks, WhyStop ] = ...
    optMAC_DA(data);


%% post-process result
% create assignment matrix
z_s = convertResult( PRM_MAC.poss_assignments, MinRisk_assignSets);

% create binary centroids out of probabilistic ones
u_s = round(1-betas);


%% show cluster result
% rows/columns will be permuted according to the clustering result (objects
% of the same clusters are adjacent to each other, blocks of ones in 
% the centroids will be adjacent to each other).
% If you found a good clustering result, the outcome will look nicely
% structured
visualize_X2ZYfactorization( data, z_s, u_s, 'sorted', 3);





