function epsilon = update_mix_epsilon_bisect(resps, betas, r, data, T)
% bisection search for optimal epsilon param
% would be fater with Newton. might be updated one day...
% the derivative used in the subfunction is provided in the paper

global PRM_MAC;


betas = single(betas);
data = single(data);
if PRM_MAC.Mstep.eps.roundBetas
    betas = round(betas);
end
% compute proxy betas for each assignment set
betaLj = combineBetas(betas, PRM_MAC.poss_assignments);

% how close to zero shall bisection search stop?
tol = PRM_MAC.Mstep.eps.Tol;
if PRM_MAC.Mstep.eps.decreaseTol && (T<PRM_MAC.Mstep.eps.decreaseT)
    tol = tol / PRM_MAC.Mstep.eps.decreaseFac;
end

clear betas

%% bisection search
options = optimset('MaxIter', 50, 'TolX', tol);
try
    epsilon = fzero(@(x)dF_mix_epsilon_global(...
        resps, betaLj, r, data, x, PRM_MAC.Mstep.vectorisationGranul),...
        [ eps 1-eps ], options);
    
catch ME
    if strcmp(ME.identifier, 'MATLAB:fzero:ValuesAtEndPtsSameSign' )
        % no sign change in the interval [ 0; 1 ]
        % searching for smallest derivation
        
        % test for lower interval bound
        dF_0 = dF_mix_epsilon_global(resps, betaLj, r, data, eps,...
            PRM_MAC.Mstep.vectorisationGranul);
        if abs(dF_0) < tol
            epsilon = eps;
        else
            %test for upper interval bound
            dF_1 = dF_mix_epsilon_global(resps, betaLj, r, data, 1,...
                PRM_MAC.Mstep.vectorisationGranul);
            if abs(dF_1) < tol
                epsilon = 1-eps;
            else
                % pick value with smallest derivation
                if abs(dF_0) < abs(dF_1)
                    epsilon = eps;
                else
                    epsilon = 1-eps;
                end
                
            end
        end
        
    else
        % if fzero failed because of another error
        disp('update_mix_epsilon_bisec: fzero failed with error:');
        disp(ME.message);
    end
    
end

end



function dF = ...
    dF_mix_epsilon_global(resps, betaLj, r, data, epsilon, vectGran)
% computes the derivation of the free energy w.r.t. epsilon as described in
% Eq.6 of ICML paper


[ Nobj, Nfeat ] = size(data);

prefac_x0 = epsilon*(1-r)+ (1-epsilon)*(  betaLj); %x=0
prefac_x1 = epsilon*r    + (1-epsilon)*(1-betaLj); %x=1

% substitute 0s with very small number to avoid division by 0
prefac_x0( prefac_x0(:) == 0 ) = 1e-60;
prefac_x1( prefac_x1(:) == 0 ) = 1e-60;

prefac_x0 = (1 - r - betaLj) ./ prefac_x0; % Eq.6, g_{Lj0}
prefac_x1 = (r - 1 + betaLj) ./ prefac_x1; % Eq.6, g_{Lj1}

mask = repmat( data, [1 1 2]); %indicates where ones and zeros are
mask(:,:,2) = 1-mask(:,:,1);    %indicates the zeros
clear data;


dF  = zeros(1,Nfeat);
% compute granularity of vectorization
gran = round(vectGran /Nobj /size(resps,2));
restind = rem(Nfeat,gran);
if restind>0
    tmp = cat(3,resps * prefac_x1(:,Nfeat-restind+1:Nfeat), ...
        resps * prefac_x0(:,Nfeat-restind+1:Nfeat));
    dF(Nfeat-restind+1:Nfeat) = sum (sum( ...
        tmp.*mask(:,Nfeat-restind+1:Nfeat,:)   , 3 ),1 );
end

if restind<Nfeat
    for chunk = (0:floor(Nfeat/gran)-1)*gran+1
        % perform product with resps
        % this includes summing over assignment-set
        tmp = cat(3,resps * prefac_x1(:,chunk:chunk+gran-1), ...
            resps * prefac_x0(:,chunk:chunk+gran-1));
        
        % pick the x=1 and x=0 contributions and sum over objects
        dF(chunk:chunk+gran-1) = sum (sum( ...
            tmp.*mask(:,chunk:chunk+gran-1,:), 3 ),1 );
    end
end
clear mask;

% debugging:
if any(isnan( dF ))
    disp('dF_mix_epsilon_global: got NAN values in dF !');
end

% sum over features
dF = -real(sum( dF ));

end
